@@ -3,17 +3,18 @@
 import copy
 import logging
 
-from sql import Column
-
 from trytond.model import ModelView, ModelSQL, Model, fields
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond import backend
 from trytond.config import config
+from trytond.tools.multivalue import migrate_property
+from trytond.modules.company.model import (
+    CompanyMultiValueMixin, CompanyValueMixin)
 
 __all__ = ['Template', 'Product', 'price_digits', 'TemplateFunction',
-    'TemplateCategory']
+    'ProductListPrice', 'ProductCostPrice', 'TemplateCategory']
 logger = logging.getLogger(__name__)
 
 STATES = {
@@ -33,7 +34,7 @@ COST_PRICE_METHODS = [
 price_digits = (16, config.getint('product', 'price_decimal', default=4))
 
 
-class Template(ModelSQL, ModelView):
+class Template(ModelSQL, ModelView, CompanyMultiValueMixin):
     "Product Template"
     __name__ = "product.template"
     name = fields.Char('Name', size=None, required=True, translate=True,
@@ -46,12 +47,19 @@ class Template(ModelSQL, ModelView):
             'invisible': Eval('type', 'goods') != 'goods',
             },
         depends=['active', 'type'])
-    list_price = fields.Property(fields.Numeric('List Price', states=STATES,
-            digits=price_digits, depends=DEPENDS, required=True))
-    cost_price = fields.Property(fields.Numeric('Cost Price', states=STATES,
-            digits=price_digits, depends=DEPENDS, required=True))
-    cost_price_method = fields.Property(fields.Selection(COST_PRICE_METHODS,
-            'Cost Method', required=True, states=STATES, depends=DEPENDS))
+    list_price = fields.MultiValue(fields.Numeric(
+            "List Price", required=True, digits=price_digits,
+            states=STATES, depends=DEPENDS))
+    list_prices = fields.One2Many(
+        'product.list_price', 'template', "List Prices")
+    cost_price = fields.MultiValue(fields.Numeric(
+            "Cost Price", required=True, digits=price_digits,
+            states=STATES, depends=DEPENDS))
+    cost_price_method = fields.MultiValue(fields.Selection(
+            COST_PRICE_METHODS, "Cost Method", required=True,
+            states=STATES, depends=DEPENDS))
+    cost_prices = fields.One2Many(
+        'product.cost_price', 'template', "Cost Prices")
     default_uom = fields.Many2One('product.uom', 'Default UOM', required=True,
         states=STATES, depends=DEPENDS)
     default_uom_category = fields.Function(
@@ -92,6 +100,15 @@ class Template(ModelSQL, ModelView):
                 'The column "category" on table "%s" must be dropped manually',
                 cls._table)
 
+    @classmethod
+    def multivalue_model(cls, field):
+        pool = Pool()
+        if field == 'list_price':
+            return pool.get('product.list_price')
+        elif field in {'cost_price', 'cost_price_method'}:
+            return pool.get('product.cost_price')
+        return super(Template, cls).multivalue_model(field)
+
     @staticmethod
     def default_active():
         return True
@@ -104,16 +121,17 @@ class Template(ModelSQL, ModelView):
     def default_consumable():
         return False
 
+    @classmethod
+    def default_cost_price_method(cls):
+        pool = Pool()
+        Configuration = pool.get('product.configuration')
+        return Configuration(1).default_cost_price_method
+
     @staticmethod
     def default_products():
-        pool = Pool()
-        Product = pool.get('product.product')
         if Transaction().user == 0:
             return []
-        fields_names = list(f for f in Product._fields.keys()
-            if f not in ('id', 'create_uid', 'create_date',
-                'write_uid', 'write_date'))
-        return [Product.default_get(fields_names)]
+        return [{}]
 
     @fields.depends('default_uom')
     def on_change_with_default_uom_category(self, name=None):
@@ -143,6 +161,9 @@ class TemplateFunction(fields.Function):
     def __init__(self, field):
         super(TemplateFunction, self).__init__(
             field, 'get_template', searcher='search_template')
+        # Disable on_change as it is managed by on_change_template
+        self.on_change = set()
+        self.on_change_with = set()
 
     def __copy__(self):
         return TemplateFunction(copy.copy(self._field))
@@ -282,12 +303,15 @@ class Product(ModelSQL, ModelView):
         field = name[:-4]
         if Transaction().context.get('uom'):
             to_uom = Uom(Transaction().context['uom'])
-            for product in products:
-                res[product.id] = Uom.compute_price(
-                    product.default_uom, getattr(product, field), to_uom)
         else:
-            for product in products:
-                res[product.id] = getattr(product, field)
+            to_uom = None
+        for product in products:
+            price = getattr(product, field)
+            if to_uom and product.default_uom.category == to_uom.category:
+                res[product.id] = Uom.compute_price(
+                    product.default_uom, price, to_uom)
+            else:
+                res[product.id] = price
         return res
 
     @classmethod
@@ -297,6 +321,72 @@ class Product(ModelSQL, ModelView):
             yield id_, rec_name, icon
 
 
+class ProductListPrice(ModelSQL, CompanyValueMixin):
+    "Product List Price"
+    __name__ = 'product.list_price'
+    template = fields.Many2One(
+        'product.template', "Template", ondelete='CASCADE', select=True)
+    list_price = fields.Numeric("List Price", digits=price_digits)
+
+    @classmethod
+    def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
+        exist = TableHandler.table_exist(cls._table)
+
+        super(ProductListPrice, cls).__register__(module_name)
+
+        if not exist:
+            cls._migrate_property([], [], [])
+
+    @classmethod
+    def _migrate_property(cls, field_names, value_names, fields):
+        field_names.append('list_price')
+        value_names.append('list_price')
+        fields.append('company')
+        migrate_property(
+            'product.template', field_names, cls, value_names,
+            parent='template', fields=fields)
+
+
+class ProductCostPrice(ModelSQL, CompanyValueMixin):
+    "Product Cost Price"
+    __name__ = 'product.cost_price'
+    template = fields.Many2One(
+        'product.template', "Template", ondelete='CASCADE', select=True)
+    cost_price = fields.Numeric(
+        "Cost Price", digits=price_digits)
+    cost_price_method = fields.Selection(
+        'get_cost_price_methods', "Cost Method")
+
+    @classmethod
+    def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
+        exist = TableHandler.table_exist(cls._table)
+
+        super(ProductCostPrice, cls).__register__(module_name)
+
+        if not exist:
+            cls._migrate_property([], [], [])
+
+    @classmethod
+    def _migrate_property(cls, field_names, value_names, fields):
+        field_names.extend(['cost_price', 'cost_price_method'])
+        value_names.extend(['cost_price', 'cost_price_method'])
+        fields.append('company')
+        migrate_property(
+            'product.template', field_names, cls, value_names,
+            parent='template', fields=fields)
+
+    @classmethod
+    def get_cost_price_methods(cls):
+        pool = Pool()
+        Template = pool.get('product.template')
+        field_name = 'cost_price_method'
+        methods = Template.fields_get([field_name])[field_name]['selection']
+        methods.append((None, ''))
+        return methods
+
+
 class TemplateCategory(ModelSQL):
     'Template - Category'
     __name__ = 'product.template-product.category'
